/* 
 *========================================================================
 * 
 * Copyright 2001-2004 Vincent Massol & Matt Smith.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *========================================================================
 */
package patterntesting.avalon;

import org.apache.avalon.framework.thread.ThreadSafe;

/**
 * Pattern Test that verifies that <code>ThreadSafe</code> Avalon
 * component can only set instance variables in their <code>compose</code>
 * <code>initialize()</code>, <code>dispose()</code> or 
 * <code>configure()</code> methods. This is to prevent setting state in 
 * a <code>ThreadSafe</code> component.
 *
 * @author <a href="mailto:sdonagh@users.sourceforge.net">Steve Donagh</a>
 * @author <a href="mailto:vmassol@apache.org">Vincent Massol</a>
 *
 * @version $Id: AbstractThreadSafeTest.aj,v 1.1 2007/05/22 20:38:51 oboehm Exp $
 */
public abstract aspect AbstractThreadSafeTest
{
    /**
     * Specify what is application code that should be subject to the
     * pattern test.
     * 
     * Ex: <code>pointcut applicationCode(): within(patterntesting.*)</code>
     */
    public abstract pointcut applicationCode();

    /**
     * Specify which code should be excluded from the pattern test.
     */
    public abstract pointcut allowedCode();

    /**
     * Valid locations in ThreadSafe components where instance variables
     * can be set.
     * 
     * Note: The withincode() should be replaced by cflow() to be correct
     * but cflow() is a dynamic PCD, so it'll have to wait for support 
     * for runtime aspects in the PatternTesting project for that.
     */
    pointcut validLocations() :
        withincode(public * ThreadSafe+.dispose(..))
        || withincode(public * ThreadSafe+.compose(..))
        || withincode(public * ThreadSafe+.initialize(..))
        || withincode(public * ThreadSafe+.configure(..));

    /**
     * Instance variable initializations. They are valid as they are
     * initialized at instance creation time.
     */
    pointcut instanceInitializations() :
        set(* *) && !withincode(* *(..));

    /**
     * Specify which calls are forbidden.
     */
    pointcut forbiddenCalls() :
        set(!ThreadSafe+ ThreadSafe+.*)
        && !instanceInitializations()
        && !validLocations();
    
    /**
     * Raise error if violations found.
     */
    declare error: (applicationCode() && forbiddenCalls()) && !allowedCode() :
        "ThreadSafe components should not set state outside their compose(), initialize(), dispose() or configure() methods";
} 