/**
 * $Id: AbstractNullPointerTrap.aj,v 1.4 2008/04/27 16:56:58 oboehm Exp $
 *
 * Copyright (c) 2007 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 11.12.2007 by oliver (ob@oasd.de)
 */
package patterntesting.java.runtime;

import patterntesting.java.runtime.annotation.*;

/**
 * What is the difference to the AbstractNullTest aspect?
 * In the AbstractNullTest aspect we log only if a null value is given to a
 * method. The NullPointerTrap here is much stricter: it <b>requires</b>
 * that
 * <ul>
 * 	<li>no null value is given</li>
 * 	<li>null as return value is forbidden</li>
 * 	<li>if one of this requirements are hurt an AssertionExecption is
 * 		thrown.
 * 	</li>
 * </ul>
 * As in AbstractNullTest you can define exceptions from these strict rules
 * by defining the abstract pointcuts <i>applicationCode</i>,
 * <i>nullArgsAllowed</li> and <i>mayReturnNull</li>.
 * 
 * This aspect is intended more for finding bugs in the test phase whereas
 * AbstractNullTest can be also used during production because it logs only
 * the problematic calls with a null parameter.
 * 
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 11.12.2007
 * @version $Revision: 1.4 $
 * 
 * @see AbstractNullTestTest
 */
public abstract aspect AbstractNullPointerTrap {
	
	/**
	 * Specify what the application code is that should be subject to the
	 * pattern test.
	 * <br/>
	 * Ex: <code>pointcut applicationCode(): within(patterntesting.sample.*)</code>
	 */
	public abstract pointcut applicationCode();

	/**
	 * Specify which code is allowed to have <code>null</code> parameters.
	 */
	public abstract pointcut nullArgsAllowed();

	/**
	 * Specify which methods may return <code>null</code>.
	 * Some methods (like find methods or getters) may return a null value.
	 */
	public abstract pointcut mayReturnNull();
	
	
	
	/**
	 * Addresses all methods (or constructors) with at least one argument with should
	 * not be given as null parameter
	 */
	pointcut invalidNullParameter() :
		(execution(* *..*(*, ..)) || execution(*..new(*, ..)))
		&& applicationCode()
		&& !nullArgsAllowed()
		&& !(execution(@NullArgsAllowed * *..*(*, ..))
				|| execution(@NullArgsAllowed *..new(*, ..)))
		&& !within(AbstractNullPointerTrap);
	
	/**
	 * If a (null) parameter is detected something is wrong with the
	 * application code
	 */
	before() : invalidNullParameter() {
		Object[] args = thisJoinPoint.getArgs();
		if (args != null) {
			for(int i = 0; i < args.length; i++) {
				if (args[i] == null) {
					throw new AssertionError("arg" + i + " is null");
				}
			}
		}
	}
	
	
	
	/**
	 * To check all non void methods if returns a null value we first define
	 * the pointcut here.
	 */
	private pointcut nonVoidMethods() :
		execution(Object+ *..*(..))
		&& applicationCode()
		&& !mayReturnNull()
		&& !(execution(@MayReturnNull Object+ *..*(..)))
		&& !within(AbstractNullPointerTrap);
	
	/**
	 * A 'null' as return value is often an indication that something is wrong
	 * (otherwise an exception should be thrown)
	 */
    after() returning(Object returned) : nonVoidMethods() {
        if (returned == null) {
            throw new AssertionError(thisJoinPoint.getSignature() 
                    + " returns (null)!");
        }
    }

}

/**
 * $Log: AbstractNullPointerTrap.aj,v $
 * Revision 1.4  2008/04/27 16:56:58  oboehm
 * lot of JavaDocs added
 *
 * Revision 1.3  2008/04/27 11:50:55  oboehm
 * methods can now be marked as @MayReturnNull
 *
 * Revision 1.2  2008/04/25 06:49:50  oboehm
 * @NullArgsAllowed and @MayReturnNull added to be able to mark methods
 *
 * Revision 1.1  2007/12/11 19:32:39  oboehm
 * AbstractNullPointerTrap with test suite added
 *
 * $Source: /cvsroot/patterntesting/PatternTesting05/src/main/aspect/patterntesting/java/runtime/AbstractNullPointerTrap.aj,v $
 */
